@@ -12,7 +12,7 @@ implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the GNU Library General Public License for more details.
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free Software
-Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 /* 
@@ -29,9 +29,7 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <new.h>
 #include <builtin.h>
 
-// extern "C" {
-#include <regex.h>
-// }
+#undef OK
 
 void String::error(const char* msg) const
 {
@@ -133,7 +131,7 @@ inline static StrRep* Snew(int newsiz)
   if (allocsiz >= MAXStrRep_SIZE)
     (*lib_error_handler)("String", "Requested length out of range");
     
-  StrRep* rep = (StrRep *) new char[allocsiz];
+  StrRep* rep = new (operator new (allocsiz)) StrRep;
   rep->sz = allocsiz - sizeof(StrRep);
   return rep;
 }
@@ -171,7 +169,8 @@ StrRep* Salloc(StrRep* old, const char* src, int srclen, int newlen)
 // generally be faster in the long run to get new space & copy
 // than to call realloc
 
-StrRep* Sresize(StrRep* old, int newlen)
+static StrRep*
+Sresize(StrRep* old, int newlen)
 {
   if (old == &_nilStrRep) old = 0;
   StrRep* rep;
@@ -191,9 +190,17 @@ StrRep* Sresize(StrRep* old, int newlen)
   return rep;
 }
 
+void
+String::alloc (int newsize)
+{
+  unsigned short old_len = rep->len;
+  rep = Sresize(rep, newsize);
+  rep->len = old_len;
+}
+
 // like allocate, but we know that src is a StrRep
 
-StrRep* Scopy(StrRep* old, StrRep* s)
+StrRep* Scopy(StrRep* old, const StrRep* s)
 {
   if (old == &_nilStrRep) old = 0;
   if (s == &_nilStrRep) s = 0;
@@ -319,7 +326,7 @@ inline static int scmp(const char* a, const char* b)
     return *a != 0;
   else
   {
-    signed char diff = 0;
+    int diff = 0;
     while ((diff = *a - *b++) == 0 && *a++ != 0);
     return diff;
   }
@@ -329,7 +336,7 @@ inline static int scmp(const char* a, const char* b)
 inline static int ncmp(const char* a, int al, const char* b, int bl)
 {
   int n = (al <= bl)? al : bl;
-  signed char diff;
+  int diff;
   while (n-- > 0) if ((diff = *a++ - *b++) != 0) return diff;
   return al - bl;
 }
@@ -341,7 +348,7 @@ int fcompare(const String& x, const String& y)
   int al = x.length();
   int bl = y.length();
   int n = (al <= bl)? al : bl;
-  signed char diff = 0;
+  int diff = 0;
   while (n-- > 0)
   {
     char ac = *a++;
@@ -395,7 +402,7 @@ int compare(const SubString& x, const char* b)
   {
     const char* a = x.chars();
     int n = x.length();
-    signed char diff;
+    int diff;
     while (n-- > 0) if ((diff = *a++ - *b++) != 0) return diff;
     return (*b == 0) ? 0 : -1;
   }
@@ -483,7 +490,7 @@ int String::match(int start, int sl, int exact, const char* t, int tl) const
   return tl;
 }
 
-void SubString::assign(StrRep* ysrc, const char* ys, int ylen)
+void SubString::assign(const StrRep* ysrc, const char* ys, int ylen)
 {
   if (&S == &_nilString) return;
 
@@ -959,64 +966,47 @@ int split(const String& src, String results[], int n, const Regex& r)
 }
 
 
-#if defined(__GNUG__) && !defined(NO_NRV)
-
-String join(String src[], int n, const String& separator) return x;
-{
-  String sep = separator;
-  int xlen = 0;
-  for (int i = 0; i < n; ++i)
-    xlen += src[i].length();
-  xlen += (n - 1) * sep.length();
-
-  x.alloc(xlen);
-
-  int j = 0;
-  
-  for (i = 0; i < n - 1; ++i)
-  {
-    ncopy(src[i].chars(), &(x.rep->s[j]), src[i].length());
-    j += src[i].length();
-    ncopy(sep.chars(), &(x.rep->s[j]), sep.length());
-    j += sep.length();
-  }
-  ncopy0(src[i].chars(), &(x.rep->s[j]), src[i].length());
-}
-
-#else
-
-String join(String src[], int n, const String& separator)
-{
-  String x;
-  String sep = separator;
-  int xlen = 0;
-  for (int i = 0; i < n; ++i)
-    xlen += src[i].length();
-  xlen += (n - 1) * sep.length();
-
-  x.alloc(xlen);
-
-  int j = 0;
-  
-  for (i = 0; i < n - 1; ++i)
-  {
-    ncopy(src[i].chars(), &(x.rep->s[j]), src[i].length());
-    j += src[i].length();
-    ncopy(sep.chars(), &(x.rep->s[j]), sep.length());
-    j += sep.length();
-  }
-  ncopy0(src[i].chars(), &(x.rep->s[j]), src[i].length());
-  return x;
-}
-
+#if defined(__GNUG__) && !defined(_G_NO_NRV)
+#define RETURN(r) return
+#define RETURNS(r) return r;
+#define RETURN_OBJECT(TYPE, NAME) /* nothing */
+#else /* _G_NO_NRV */
+#define RETURN(r) return r
+#define RETURNS(r) /* nothing */
+#define RETURN_OBJECT(TYPE, NAME) TYPE NAME;
 #endif
+
+String join(String src[], int n, const String& separator) RETURNS(x)
+{
+  RETURN_OBJECT(String,x)
+  String sep = separator;
+  int xlen = 0;
+  int i;
+  for (i = 0; i < n; ++i)
+    xlen += src[i].length();
+  xlen += (n - 1) * sep.length();
+
+  x.rep = Sresize (x.rep, xlen);
+
+  int j = 0;
+  
+  for (i = 0; i < n - 1; ++i)
+  {
+    ncopy(src[i].chars(), &(x.rep->s[j]), src[i].length());
+    j += src[i].length();
+    ncopy(sep.chars(), &(x.rep->s[j]), sep.length());
+    j += sep.length();
+  }
+  ncopy0(src[i].chars(), &(x.rep->s[j]), src[i].length());
+  RETURN(x);
+}
   
 /*
  misc
 */
 
     
-StrRep* Sreverse(StrRep* src, StrRep* dest)
+StrRep* Sreverse(const StrRep* src, StrRep* dest)
 {
   int n = src->len;
   if (src != dest)
@@ -1036,7 +1026,7 @@ StrRep* Sreverse(StrRep* src, StrRep* dest)
 }
 
 
-StrRep* Supcase(StrRep* src, StrRep* dest)
+StrRep* Supcase(const StrRep* src, StrRep* dest)
 {
   int n = src->len;
   if (src != dest) dest = Salloc(dest, src->s, n, n);
@@ -1046,7 +1036,7 @@ StrRep* Supcase(StrRep* src, StrRep* dest)
   return dest;
 }
 
-StrRep* Sdowncase(StrRep* src, StrRep* dest)
+StrRep* Sdowncase(const StrRep* src, StrRep* dest)
 {
   int n = src->len;
   if (src != dest) dest = Salloc(dest, src->s, n, n);
@@ -1056,7 +1046,7 @@ StrRep* Sdowncase(StrRep* src, StrRep* dest)
   return dest;
 }
 
-StrRep* Scapitalize(StrRep* src, StrRep* dest)
+StrRep* Scapitalize(const StrRep* src, StrRep* dest)
 {
   int n = src->len;
   if (src != dest) dest = Salloc(dest, src->s, n, n);
@@ -1077,7 +1067,9 @@ StrRep* Scapitalize(StrRep* src, StrRep* dest)
       {
         if (isupper(*p))
           *p = tolower(*p);
-        else if (!islower(*p) && !isdigit(*p))
+	/* A '\'' does not break a word, so that "Nathan's" stays
+	   "Nathan's" rather than turning into "Nathan'S". */
+        else if (!islower(*p) && !isdigit(*p) && (*p != '\''))
           break;
       }
     }
@@ -1085,7 +1077,7 @@ StrRep* Scapitalize(StrRep* src, StrRep* dest)
   return dest;
 }
 
-#if defined(__GNUG__) && !defined(NO_NRV)
+#if defined(__GNUG__) && !defined(_G_NO_NRV)
 
 String replicate(char c, int n) return w;
 {
@@ -1117,7 +1109,8 @@ String common_prefix(const String& x, const String& y, int startpos) return r;
   const char* topx = &(xchars[x.length()]);
   const char* ys = &(ychars[startpos]);
   const char* topy = &(ychars[y.length()]);
-  for (int l = 0; xs < topx && ys < topy && *xs++ == *ys++; ++l);
+  int l;
+  for (l = 0; xs < topx && ys < topy && *xs++ == *ys++; ++l);
   r.rep = Salloc(r.rep, ss, l, l);
 }
 
@@ -1129,7 +1122,8 @@ String common_suffix(const String& x, const String& y, int startpos) return r;
   const char* botx = xchars;
   const char* ys = &(ychars[y.length() + startpos]);
   const char* boty = ychars;
-  for (int l = 0; xs >= botx && ys >= boty && *xs == *ys ; --xs, --ys, ++l);
+  int l;
+  for (l = 0; xs >= botx && ys >= boty && *xs == *ys ; --xs, --ys, ++l);
   r.rep = Salloc(r.rep, ++xs, l, l);
 }
 
@@ -1170,7 +1164,8 @@ String common_prefix(const String& x, const String& y, int startpos)
   const char* topx = &(xchars[x.length()]);
   const char* ys = &(ychars[startpos]);
   const char* topy = &(ychars[y.length()]);
-  for (int l = 0; xs < topx && ys < topy && *xs++ == *ys++; ++l);
+  int l;
+  for (l = 0; xs < topx && ys < topy && *xs++ == *ys++; ++l);
   r.rep = Salloc(r.rep, ss, l, l);
   return r;
 }
@@ -1184,7 +1179,8 @@ String common_suffix(const String& x, const String& y, int startpos)
   const char* botx = xchars;
   const char* ys = &(ychars[y.length() + startpos]);
   const char* boty = ychars;
-  for (int l = 0; xs >= botx && ys >= boty && *xs == *ys ; --xs, --ys, ++l);
+  int l;
+  for (l = 0; xs >= botx && ys >= boty && *xs == *ys ; --xs, --ys, ++l);
   r.rep = Salloc(r.rep, ++xs, l, l);
   return r;
 }
