@@ -1,5 +1,5 @@
 /* 
-Copyright (C) 1993 Free Software Foundation
+Copyright (C) 1993, 1994 Free Software Foundation
 
 This file is part of the GNU IO Library.  This library is free
 software; you can redistribute it and/or modify it under the
@@ -13,8 +13,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU CC; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+along with this library; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 As a special exception, if you link this library with files
 compiled with a GNU compiler to produce an executable, this does not cause
@@ -23,7 +23,7 @@ This exception does not however invalidate any other reasons why
 the executable file might be covered by the GNU General Public License. */
 
 #include <libioP.h>
-#ifdef USE_DTOA
+#ifdef _IO_USE_DTOA
 /****************************************************************
  *
  * The author of this software is David M. Gay.
@@ -169,6 +169,13 @@ Exactly one of IEEE_8087, IEEE_MC68k, VAX, or IBM should be defined.
 #endif
 #endif
 
+typedef _G_uint32_t unsigned32;
+
+union doubleword {
+  double d;
+  unsigned32 u[2];
+};
+
 #ifdef IEEE_8087
 #define HIWORD 1
 #define LOWORD 0
@@ -182,10 +189,7 @@ Exactly one of IEEE_8087, IEEE_MC68k, VAX, or IBM should be defined.
 static int HIWORD = -1, LOWORD;
 static void test_endianness()
 {
-    union doubleword {
-	double d;
-	unsigned long u[2];
-    } dw;
+    union doubleword dw;
     dw.d = 10;
     if (dw.u[0] != 0) /* big-endian */
 	HIWORD=0, LOWORD=1;
@@ -197,17 +201,27 @@ static void test_endianness()
 #endif
 
 #if 0
-union {
-  double d;
-  unsigned long x[2];
-} _temp;
+union doubleword _temp;
 #endif
-#define word0(x) ((unsigned long *)&x)[HIWORD]
-#if 0
-#define word0(X) (_temp.d = X, _temp.x[HIWORD])
-#define setword0(D,X) (_temp.d = D, _temp.x[HIWORD] = X, D = _temp.d)
+#ifdef __GNUC__
+#define word0(x) ({ union doubleword _du; _du.d = (x); _du.u[HIWORD]; })
+#define word1(x) ({ union doubleword _du; _du.d = (x); _du.u[LOWORD]; })
+#define setword0(D,W) \
+  ({ union doubleword _du; _du.d = (D); _du.u[HIWORD]=(W); (D)=_du.d; })
+#define setword1(D,W) \
+  ({ union doubleword _du; _du.d = (D); _du.u[LOWORD]=(W); (D)=_du.d; })
+#define setwords(D,W0,W1) ({ union doubleword _du; \
+  _du.u[HIWORD]=(W0); _du.u[LOWORD]=(W1); (D)=_du.d; })
+#define addword0(D,W) \
+  ({ union doubleword _du; _du.d = (D); _du.u[HIWORD]+=(W); (D)=_du.d; })
+#else
+#define word0(x) ((unsigned32 *)&x)[HIWORD]
+#define word1(x) ((unsigned32 *)&x)[LOWORD]
+#define setword0(D,W) word0(D) = (W)
+#define setword1(D,W) word1(D) = (W)
+#define setwords(D,W0,W1) (setword0(D,W0),setword1(D,W1))
+#define addword0(D,X) (word0(D) += (X))
 #endif
-#define word1(x) ((unsigned long *)&x)[LOWORD]
 
 /* The following definition of Storeinc is appropriate for MIPS processors. */
 #if defined(IEEE_8087) + defined(VAX)
@@ -340,11 +354,11 @@ struct Bigint {
   short on_stack;	/* 1 if stack-allocated. */
   short sign;		/* 0 if value is positive or zero; 1 if negative. */
   int wds;		/* Current length. */
-  unsigned long x[1<<BIGINT_MINIMUM_K]; /* Actually: x[maxwds] */
+  unsigned32 x[1<<BIGINT_MINIMUM_K]; /* Actually: x[maxwds] */
 };
 
 #define BIGINT_HEADER_SIZE \
-  (sizeof(Bigint) - (1<<BIGINT_MINIMUM_K) * sizeof(unsigned long))
+  (sizeof(Bigint) - (1<<BIGINT_MINIMUM_K) * sizeof(unsigned32))
 
 typedef struct Bigint Bigint;
 
@@ -383,7 +397,7 @@ Balloc
 
   x = 1 << k;
   rv = (Bigint *)
-    malloc(BIGINT_HEADER_SIZE + x * sizeof(unsigned long));
+    malloc(BIGINT_HEADER_SIZE + x * sizeof(unsigned32));
   rv->k = k;
   rv->maxwds = x;
   rv->sign = rv->wds = 0;
@@ -411,7 +425,7 @@ Bcopy
         (Bigint *x, Bigint *y)
 #endif
 {
-  register unsigned long *xp, *yp;
+  register unsigned32 *xp, *yp;
   register int i = y->wds;
   x->sign = y->sign;
   x->wds = i;
@@ -454,8 +468,8 @@ multadd
 #endif
 {
         int i, wds;
-        unsigned long *x, y;
-        unsigned long xi, z;
+        unsigned32 *x, y;
+        unsigned32 xi, z;
 
         wds = b->wds;
         x = b->x;
@@ -481,13 +495,13 @@ static Bigint *
 s2b
 #ifdef KR_headers
         (result, s, nd0, nd, y9)
-	Bigint *result; CONST char *s; int nd0, nd; unsigned long y9;
+	Bigint *result; CONST char *s; int nd0, nd; unsigned32 y9;
 #else
-        (Bigint *result, CONST char *s, int nd0, int nd, unsigned long y9)
+        (Bigint *result, CONST char *s, int nd0, int nd, unsigned32 y9)
 #endif
 {
   int i, k;
-  long x, y;
+  _G_int32_t x, y;
 
   x = (nd + 8) / 9;
   for(k = 0, y = 1; x > y; y <<= 1, k++) ;
@@ -514,9 +528,9 @@ s2b
 static int
 hi0bits
 #ifdef KR_headers
-        (x) register unsigned long x;
+        (x) register unsigned32 x;
 #else
-        (register unsigned long x)
+        (register unsigned32 x)
 #endif
 {
         register int k = 0;
@@ -548,13 +562,13 @@ hi0bits
 static int
 lo0bits
 #ifdef KR_headers
-        (y) unsigned long *y;
+        (y) unsigned32 *y;
 #else
-        (unsigned long *y)
+        (unsigned32 *y)
 #endif
 {
         register int k;
-        register unsigned long x = *y;
+        register unsigned32 x = *y;
 
         if (x & 7) {
                 if (x & 1)
@@ -618,9 +632,9 @@ mult
 #endif
 {
         int k, wa, wb, wc;
-        unsigned long carry, y, z;
-        unsigned long *x, *xa, *xae, *xb, *xbe, *xc, *xc0;
-        unsigned long z2;
+        unsigned32 carry, y, z;
+        unsigned32 *x, *xa, *xae, *xb, *xbe, *xc, *xc0;
+        unsigned32 z2;
         if (a->wds < b->wds) {
                 Bigint *tmp = a;
                 a = b;
@@ -641,7 +655,7 @@ mult
         xbe = xb + wb;
         xc0 = c->x;
         for(; xb < xbe; xb++, xc0++) {
-                if (y = *xb & 0xffff) {
+                if ((y = *xb & 0xffff)) {
                         x = xa;
                         xc = xc0;
                         carry = 0;
@@ -655,7 +669,7 @@ mult
                                 while(x < xae);
                         *xc = carry;
                         }
-                if (y = *xb >> 16) {
+                if ((y = *xb >> 16)) {
                         x = xa;
                         xc = xc0;
                         carry = 0;
@@ -708,7 +722,7 @@ lshift
 #endif
 {
   int i;
-  unsigned long *x, *x1, *xe;
+  unsigned32 *x, *x1, *xe;
   int old_wds = b->wds;
   int n = k >> 5;
   int k1 = b->k;
@@ -726,12 +740,12 @@ lshift
   x1 = x + n; /* Destination pointer */
   if (k &= 0x1f) {
     int k1 = 32 - k;
-    unsigned long z = *--x;
+    unsigned32 z = *--x;
     if ((*x1 = (z >> k1)) != 0) {
       ++n1;
     }
     while (x > xe) {
-      unsigned long w = *--x;
+      unsigned32 w = *--x;
       *--x1 = (z << k) | (w >> k1);
       z = w;
     }
@@ -755,7 +769,7 @@ cmp
         (Bigint *a, Bigint *b)
 #endif
 {
-        unsigned long *xa, *xa0, *xb, *xb0;
+        unsigned32 *xa, *xa0, *xb, *xb0;
         int i, j;
 
         i = a->wds;
@@ -792,9 +806,9 @@ diff
 #endif
 {
         int i, wa, wb;
-        long borrow, y; /* We need signed shifts here. */
-        unsigned long *xa, *xae, *xb, *xbe, *xc;
-        long z;
+        _G_int32_t borrow, y; /* We need signed shifts here. */
+        unsigned32 *xa, *xae, *xb, *xbe, *xc;
+        _G_int32_t z;
 
         i = cmp(a,b);
         if (!i) {
@@ -854,7 +868,7 @@ ulp
         (double x)
 #endif
 {
-        register long L;
+        register _G_int32_t L;
         double a;
 
         L = (word0(x) & Exp_mask) - (P-1)*Exp_msk1;
@@ -864,20 +878,16 @@ ulp
 #ifdef IBM
                 L |= Exp_msk1 >> 4;
 #endif
-                word0(a) = L;
-                word1(a) = 0;
+                setwords(a, L, 0);
 #ifndef Sudden_Underflow
                 }
         else {
                 L = -L >> Exp_shift;
-                if (L < Exp_shift) {
-                        word0(a) = 0x80000 >> L;
-                        word1(a) = 0;
-                        }
+                if (L < Exp_shift)
+                        setwords(a, 0x80000 >> L, 0);
                 else {
-                        word0(a) = 0;
                         L -= Exp_shift;
-                        word1(a) = L >= 31 ? 1 : 1 << 31 - L;
+                        setwords(a, 0, L >= 31 ? 1 : 1 << (31 - L));
                         }
                 }
 #endif
@@ -892,15 +902,10 @@ b2d
         (Bigint *a, int *e)
 #endif
 {
-        unsigned long *xa, *xa0, w, y, z;
+        unsigned32 *xa, *xa0, w, y, z;
         int k;
         double d;
-#ifdef VAX
-        unsigned long d0, d1;
-#else
-#define d0 word0(d)
-#define d1 word1(d)
-#endif
+        unsigned32 d0, d1;
 
         xa0 = a->x;
         xa = xa0 + a->wds;
@@ -911,16 +916,16 @@ b2d
         k = hi0bits(y);
         *e = 32 - k;
         if (k < Ebits) {
-                d0 = Exp_1 | y >> Ebits - k;
+                d0 = Exp_1 | y >> (Ebits - k);
                 w = xa > xa0 ? *--xa : 0;
-                d1 = y << (32-Ebits) + k | w >> Ebits - k;
+                d1 = y << ((32-Ebits) + k) | w >> (Ebits - k);
                 goto ret_d;
                 }
         z = xa > xa0 ? *--xa : 0;
         if (k -= Ebits) {
-                d0 = Exp_1 | y << k | z >> 32 - k;
+                d0 = Exp_1 | y << k | z >> (32 - k);
                 y = xa > xa0 ? *--xa : 0;
-                d1 = z << k | y >> 32 - k;
+                d1 = z << k | y >> (32 - k);
                 }
         else {
                 d0 = Exp_1 | y;
@@ -928,11 +933,9 @@ b2d
                 }
  ret_d:
 #ifdef VAX
-        word0(d) = d0 >> 16 | d0 << 16;
-        word1(d) = d1 >> 16 | d1 << 16;
+        setwords(d, d0 >> 16 | d0 << 16, d1 >> 16 | d1 << 16);
 #else
-#undef d0
-#undef d1
+	setwords (d, d0, d1);
 #endif
         return d;
         }
@@ -946,14 +949,14 @@ d2b
 #endif
 {
         int de, i, k;
-        unsigned long *x, y, z;
+        unsigned32 *x, y, z;
+        unsigned32 d0, d1;
 #ifdef VAX
-        unsigned long d0, d1;
         d0 = word0(d) >> 16 | word0(d) << 16;
         d1 = word1(d) >> 16 | word1(d) << 16;
 #else
-#define d0 word0(d)
-#define d1 word1(d)
+	d0 = word0(d);
+	d1 = word1(d);
 #endif
 
         result = Brealloc(result, 1);
@@ -972,9 +975,9 @@ d2b
 	  z |= Exp_msk11;
 #endif
 
-        if (y = d1) {
-                if (k = lo0bits(&y)) {
-                        x[0] = y | z << 32 - k;
+        if ((y = d1)) {
+                if ((k = lo0bits(&y))) {
+                        x[0] = y | z << (32 - k);
                         z >>= k;
                         }
                 else
@@ -1010,8 +1013,6 @@ d2b
 #endif
         return result;
         }
-#undef d0
-#undef d1
 
 static double
 ratio
@@ -1029,28 +1030,28 @@ ratio
         k = ka - kb + 32*(a->wds - b->wds);
 #ifdef IBM
         if (k > 0) {
-                word0(da) += (k >> 2)*Exp_msk1;
+                addword0(da, (k >> 2)*Exp_msk1);
                 if (k &= 3)
                         da *= 1 << k;
                 }
         else {
                 k = -k;
-                word0(db) += (k >> 2)*Exp_msk1;
+                addword0(db,(k >> 2)*Exp_msk1);
                 if (k &= 3)
                         db *= 1 << k;
                 }
 #else
         if (k > 0)
-                word0(da) += k*Exp_msk1;
+                addword0(da, k*Exp_msk1);
         else {
                 k = -k;
-                word0(db) += k*Exp_msk1;
+                addword0(db, k*Exp_msk1);
                 }
 #endif
         return da / db;
         }
 
-static double
+static CONST double
 tens[] = {
                 1e0, 1e1, 1e2, 1e3, 1e4, 1e5, 1e6, 1e7, 1e8, 1e9,
                 1e10, 1e11, 1e12, 1e13, 1e14, 1e15, 1e16, 1e17, 1e18, 1e19,
@@ -1060,19 +1061,18 @@ tens[] = {
 #endif
                 };
 
-static double
 #ifdef IEEE_Arith
-bigtens[] = { 1e16, 1e32, 1e64, 1e128, 1e256 };
-static double tinytens[] = { 1e-16, 1e-32, 1e-64, 1e-128, 1e-256 };
+static CONST double bigtens[] = { 1e16, 1e32, 1e64, 1e128, 1e256 };
+static CONST double tinytens[] = { 1e-16, 1e-32, 1e-64, 1e-128, 1e-256 };
 #define n_bigtens 5
 #else
 #ifdef IBM
-bigtens[] = { 1e16, 1e32, 1e64 };
-static double tinytens[] = { 1e-16, 1e-32, 1e-64 };
+static CONST double bigtens[] = { 1e16, 1e32, 1e64 };
+static CONST double tinytens[] = { 1e-16, 1e-32, 1e-64 };
 #define n_bigtens 3
 #else
-bigtens[] = { 1e16, 1e32 };
-static double tinytens[] = { 1e-16, 1e-32 };
+static CONST double bigtens[] = { 1e16, 1e32 };
+static CONST double tinytens[] = { 1e-16, 1e-32 };
 #define n_bigtens 2
 #endif
 #endif
@@ -1089,8 +1089,8 @@ _IO_strtod
                  e, e1, esign, i, j, k, nd, nd0, nf, nz, nz0, sign;
         CONST char *s, *s0, *s1;
         double aadj, aadj1, adj, rv, rv0;
-        long L;
-        unsigned long y, z;
+        _G_int32_t L;
+        unsigned32 y, z;
 	Bigint _bb, _b_avail, _bd, _bd0, _bs, _delta;
 	Bigint *bb = Binit(&_bb);
 	Bigint *bd = Binit(&_bd);
@@ -1102,6 +1102,7 @@ _IO_strtod
 	TEST_ENDIANNESS;
         sign = nz0 = nz = 0;
         rv = 0.;
+	(void)&rv;		/* Force rv into the stack */
         for(s = s00;;s++) switch(*s) {
                 case '-':
                         sign = 1;
@@ -1111,6 +1112,8 @@ _IO_strtod
                                 goto break2;
                         /* no break */
                 case 0:
+			/* "+" and "-" should be reported as an error? */
+			sign = 0;
 			s = s00;
                         goto ret;
                 case '\t':
@@ -1252,12 +1255,12 @@ _IO_strtod
                                  * worry about overflow here...
                                  */
  vax_ovfl_check:
-                                word0(rv) -= P*Exp_msk1;
+                                addword0(rv, - P*Exp_msk1);
                                 /* rv = */ rounded_product(rv, tens[e]);
                                 if ((word0(rv) & Exp_mask)
                                  > Exp_msk1*(DBL_MAX_EXP+Bias-1-P))
                                         goto ovfl;
-                                word0(rv) += P*Exp_msk1;
+                                addword0(rv, P*Exp_msk1);
 #else
                                 /* rv = */ rounded_product(rv, tens[e]);
 #endif
@@ -1276,7 +1279,7 @@ _IO_strtod
         /* Get starting approximation = rv * 10**e1 */
 
         if (e1 > 0) {
-                if (i = e1 & 15)
+                if ((i = e1 & 15))
                         rv *= tens[i];
                 if (e1 &= ~15) {
                         if (e1 > DBL_MAX_10_EXP) {
@@ -1297,7 +1300,7 @@ _IO_strtod
                                         if (e1 & 1)
                                                 rv *= bigtens[j];
                         /* The last multiplication could overflow. */
-                                word0(rv) -= P*Exp_msk1;
+                                addword0(rv, -P*Exp_msk1);
                                 rv *= bigtens[j];
                                 if ((z = word0(rv) & Exp_mask)
                                  > Exp_msk1*(DBL_MAX_EXP+Bias-P))
@@ -1305,18 +1308,17 @@ _IO_strtod
                                 if (z > Exp_msk1*(DBL_MAX_EXP+Bias-1-P)) {
                                         /* set to largest number */
                                         /* (Can't trust DBL_MAX) */
-                                        word0(rv) = Big0;
-                                        word1(rv) = Big1;
+                                        setwords(rv, Big0, Big1);
                                         }
                                 else
-                                        word0(rv) += P*Exp_msk1;
+                                        addword0(rv, P*Exp_msk1);
                                 }
 
                         }
                 }
         else if (e1 < 0) {
                 e1 = -e1;
-                if (i = e1 & 15)
+                if ((i = e1 & 15))
                         rv /= tens[i];
                 if (e1 &= ~15) {
                         e1 >>= 4;
@@ -1335,8 +1337,7 @@ _IO_strtod
                                         errno = ERANGE;
                                         goto ret;
                                         }
-                                word0(rv) = Tiny0;
-                                word1(rv) = Tiny1;
+                                setwords(rv, Tiny0, Tiny1);
                                 /* The refinement below will clean
                                  * this approximation up.
                                  */
@@ -1428,13 +1429,13 @@ _IO_strtod
                                 if ((word0(rv) & Bndry_mask1) == Bndry_mask1
                                  &&  word1(rv) == 0xffffffff) {
                                         /*boundary case -- increment exponent*/
-                                        word0(rv) = (word0(rv) & Exp_mask)
-                                                + Exp_msk1
+                                        setword0(rv, (word0(rv) & Exp_mask)
+						 + Exp_msk1);
 #ifdef IBM
-                                                | Exp_msk1 >> 4
+                                        setword0 (rv,
+						  word0(rv) | (Exp_msk1 >> 4));
 #endif
-                                                ;
-                                        word1(rv) = 0;
+                                        setword1(rv, 0);
                                         break;
                                         }
                                 }
@@ -1453,8 +1454,7 @@ _IO_strtod
 #else
                                 L = (word0(rv) & Exp_mask) - Exp_msk1;
 #endif
-                                word0(rv) = L | Bndry_mask1;
-                                word1(rv) = 0xffffffff;
+                                setwords(rv, L | Bndry_mask1, 0xffffffff);
 #ifdef IBM
                                 continue;
 #else
@@ -1523,25 +1523,24 @@ _IO_strtod
 
                 if (y == Exp_msk1*(DBL_MAX_EXP+Bias-1)) {
                         rv0 = rv;
-                        word0(rv) -= P*Exp_msk1;
+                        addword0(rv, - P*Exp_msk1);
                         adj = aadj1 * ulp(rv);
                         rv += adj;
                         if ((word0(rv) & Exp_mask) >=
                                         Exp_msk1*(DBL_MAX_EXP+Bias-P)) {
                                 if (word0(rv0) == Big0 && word1(rv0) == Big1)
                                         goto ovfl;
-                                word0(rv) = Big0;
-                                word1(rv) = Big1;
+                                setwords(rv, Big0, Big1);
                                 continue;
                                 }
                         else
-                                word0(rv) += P*Exp_msk1;
+                                addword0(rv, P*Exp_msk1);
                         }
                 else {
 #ifdef Sudden_Underflow
                         if ((word0(rv) & Exp_mask) <= P*Exp_msk1) {
                                 rv0 = rv;
-                                word0(rv) += P*Exp_msk1;
+                                addword0(rv, P*Exp_msk1);
                                 adj = aadj1 * ulp(rv);
                                 rv += adj;
 #ifdef IBM
@@ -1553,12 +1552,11 @@ _IO_strtod
                                         if (word0(rv0) == Tiny0
                                          && word1(rv0) == Tiny1)
                                                 goto undfl;
-                                        word0(rv) = Tiny0;
-                                        word1(rv) = Tiny1;
+                                        setwords(rv, Tiny0, Tiny1);
                                         continue;
                                         }
                                 else
-                                        word0(rv) -= P*Exp_msk1;
+                                        addword0(rv, -P*Exp_msk1);
                                 }
                         else {
                                 adj = aadj1 * ulp(rv);
@@ -1584,7 +1582,7 @@ _IO_strtod
                 z = word0(rv) & Exp_mask;
                 if (y == z) {
                         /* Can we stop now? */
-                        L = (long)aadj;
+                        L = (_G_int32_t)aadj;
                         aadj -= L;
                         /* The tolerances below are conservative. */
                         if (dsign || word1(rv) || word0(rv) & Bndry_mask) {
@@ -1616,11 +1614,11 @@ quorem
 #endif
 {
         int n;
-        long borrow, y;
-        unsigned long carry, q, ys;
-        unsigned long *bx, *bxe, *sx, *sxe;
-        long z;
-        unsigned long si, zs;
+        _G_int32_t borrow, y;
+        unsigned32 carry, q, ys;
+        unsigned32 *bx, *bxe, *sx, *sxe;
+        _G_int32_t z;
+        unsigned32 si, zs;
 
         n = S->wds;
 #ifdef DEBUG
@@ -1773,7 +1771,7 @@ _IO_dtoa
         int bbits, b2, b5, be, dig, i, ieps, ilim, ilim0, ilim1,
                 j, j1, k, k0, k_check, leftright, m2, m5, s2, s5,
                 spec_case, try_quick;
-        long L;
+        _G_int32_t L;
 #ifndef Sudden_Underflow
         int denorm;
 #endif
@@ -1790,14 +1788,19 @@ _IO_dtoa
 
 	TEST_ENDIANNESS;
         if (result) {
+		/* result is contains a string, so its fields (interpreted
+		   as a Bigint have been trashed.  Restore them.
+		   This is a really ugly interface - result should
+		   not be static, since that is not thread-safe.  FIXME. */
                 result->k = result_k;
                 result->maxwds = 1 << result_k;
+                result->on_stack = 0;
                 }
 
         if (word0(d) & Sign_bit) {
                 /* set sign for everything, including 0's and NaNs */
                 *sign = 1;
-                word0(d) &= ~Sign_bit;  /* clear sign bit */
+                setword0(d, word0(d) & ~Sign_bit);  /* clear sign bit */
                 }
         else
                 *sign = 0;
@@ -1845,8 +1848,7 @@ _IO_dtoa
         if (i) {
 #endif
                 d2 = d;
-                word0(d2) &= Frac_mask1;
-                word0(d2) |= Exp_11;
+                setword0(d2, (word0(d2) & Frac_mask1) | Exp_11);
 #ifdef IBM
                 if (j = 11 - hi0bits(word0(d2) & Frac_mask))
                         d2 /= 1 << j;
@@ -1862,13 +1864,13 @@ _IO_dtoa
                 }
         else {
                 /* d is denormalized */
-		unsigned long x;
+		unsigned32 x;
 
                 i = bbits + be + (Bias + (P-1) - 1);
-                x = i > 32  ? word0(d) << 64 - i | word1(d) >> i - 32
-                            : word1(d) << 32 - i;
+                x = i > 32  ? word0(d) << (64 - i) | word1(d) >> (i - 32)
+                            : word1(d) << (32 - i);
                 d2 = x;
-                word0(d2) -= 31*Exp_msk1; /* adjust exponent */
+                addword0(d2, - 31*Exp_msk1); /* adjust exponent */
                 i -= (Bias + (P-1) - 1) + 1;
                 denorm = 1;
                 }
@@ -1961,11 +1963,15 @@ _IO_dtoa
                                 i = 1;
                 }
 	/* i is now an upper bound of the number of digits to generate. */
-        j = sizeof(unsigned long) * (1<<BIGINT_MINIMUM_K);
+        j = sizeof(unsigned32) * (1<<BIGINT_MINIMUM_K);
 	/* The test is <= so as to allow room for the final '\0'. */
         for(result_k = BIGINT_MINIMUM_K; BIGINT_HEADER_SIZE + j <= i;
                 j <<= 1) result_k++;
-        result = Brealloc(result, result_k);
+        if (!result || result_k > result->k)
+        {
+          Bfree (result);
+          result = Balloc(result_k);
+        }
         s = s0 = (char *)result;
 
         if (ilim >= 0 && ilim <= Quick_max && try_quick) {
@@ -1993,7 +1999,7 @@ _IO_dtoa
                                         }
                         d /= ds;
                         }
-                else if (j1 = -k) {
+                else if ((j1 = -k)) {
                         d *= tens[j1 & 0xf];
                         for(j = j1 >> 4; j; j >>= 1, i++)
                                 if (j & 1) {
@@ -2010,7 +2016,7 @@ _IO_dtoa
                         ieps++;
                         }
                 eps = ieps*d + 7.;
-                word0(eps) -= (P-1)*Exp_msk1;
+                addword0(eps, - (P-1)*Exp_msk1);
                 if (ilim == 0) {
                         d -= 5.;
                         if (d > eps)
@@ -2026,7 +2032,7 @@ _IO_dtoa
                          */
                         eps = 0.5/tens[ilim-1] - eps;
                         for(i = 0;;) {
-                                L = (long)d;
+                                L = (_G_int32_t)d;
                                 d -= L;
                                 *s++ = '0' + (int)L;
                                 if (d < eps)
@@ -2044,7 +2050,7 @@ _IO_dtoa
                         /* Generate ilim digits, then fix them up. */
                         eps *= tens[ilim-1];
                         for(i = 1;; i++, d *= 10.) {
-                                L = (long)d;
+                                L = (_G_int32_t)d;
                                 d -= L;
                                 *s++ = '0' + (int)L;
                                 if (i == ilim) {
@@ -2079,7 +2085,7 @@ _IO_dtoa
                         goto one_digit;
                         }
                 for(i = 1;; i++) {
-                        L = (long)(d / ds);
+                        L = (_G_int32_t)(d / ds);
                         d -= L*ds;
 #ifdef Check_FLT_ROUNDS
                         /* If FLT_ROUNDS == 2, L will usually be high by 1 */
@@ -2091,7 +2097,7 @@ _IO_dtoa
                         *s++ = '0' + (int)L;
                         if (i == ilim) {
                                 d += d;
-                                if (d > ds || d == ds && L & 1) {
+                                if (d > ds || (d == ds && L & 1)) {
  bump_up:
                                         while(*--s == '9')
                                                 if (s == s0) {
@@ -2156,7 +2162,7 @@ _IO_dtoa
                                 b_avail = b;
                                 b = b_tmp;
                                 }
-                        if (j = b5 - m5)
+                        if ((j = b5 - m5))
                                 b = pow5mult(b, j);
                         }
                 else
@@ -2190,7 +2196,7 @@ _IO_dtoa
          * and for all and pass them and a shift to quorem, so it
          * can do shifts and ors to compute the numerator for q.
          */
-        if (i = ((s5 ? 32 - hi0bits(S->x[S->wds-1]) : 1) + s2) & 0x1f)
+        if ((i = ((s5 ? 32 - hi0bits(S->x[S->wds-1]) : 1) + s2) & 0x1f))
                 i = 32 - i;
         if (i > 4) {
                 i -= 4;
@@ -2263,15 +2269,15 @@ _IO_dtoa
                                 goto ret;
                                 }
 #endif
-                        if (j < 0 || j == 0 && !mode
+                        if (j < 0 || (j == 0 && !mode
 #ifndef ROUND_BIASED
                                                         && !(word1(d) & 1)
 #endif
-                                        ) {
+                                        )) {
                                 if (j1 > 0) {
                                         b = lshift(b, 1);
                                         j1 = cmp(b, S);
-                                        if ((j1 > 0 || j1 == 0 && dig & 1)
+                                        if ((j1 > 0 || (j1 == 0 && dig & 1))
                                         && dig++ == '9')
                                                 goto round_9_up;
                                         }
@@ -2311,7 +2317,7 @@ _IO_dtoa
 
         b = lshift(b, 1);
         j = cmp(b, S);
-        if (j > 0 || j == 0 && dig & 1) {
+        if (j > 0 || (j == 0 && dig & 1)) {
  roundoff:
                 while(*--s == '9')
                         if (s == s0) {
@@ -2341,4 +2347,4 @@ _IO_dtoa
                 *rve = s;
         return s0;
         }
-#endif /* USE_DTOA */
+#endif /* _IO_USE_DTOA */
