@@ -13,8 +13,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU CC; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+along with this library; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 As a special exception, if you link this library with files
 compiled with a GNU compiler to produce an executable, this does not cause
@@ -33,9 +33,8 @@ the executable file might be covered by the GNU General Public License. */
 #endif
 
 _IO_FILE *
-_IO_fdopen (fd, mode)
-     int fd;
-     const char *mode;
+DEFUN(_IO_fdopen, (fd, mode),
+      int fd AND const char *mode)
 {
   int read_write;
   int posix_mode = 0;
@@ -101,9 +100,11 @@ _IO_fdopen (fd, mode)
   if (fp == NULL)
     return NULL;
   _IO_init(&fp->file, 0);
-  fp->file._jumps = &_IO_file_jumps;
+  _IO_JUMPS(&fp->file) = &_IO_file_jumps;
   _IO_file_init(&fp->file);
+#if  !_IO_UNIFIED_JUMPTABLES
   fp->vtable = NULL;
+#endif
   if (_IO_file_attach(&fp->file, fd) == NULL)
     {
       _IO_un_link(&fp->file);
