@@ -13,8 +13,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU CC; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+along with this library; see the file COPYING.  If not, write to the Free
+Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 As a special exception, if you link this library with files
 compiled with a GNU compiler to produce an executable, this does not cause
@@ -40,10 +40,10 @@ the executable file might be covered by the GNU General Public License. */
 
 //#define isspace(ch) ((ch)==' ' || (ch)=='\t' || (ch)=='\n')
 
-istream::istream(streambuf *sb, ostream* tied) : ios(sb, tied)
+istream::istream(streambuf *sb, ostream* tied)
 {
-    _flags |= ios::dont_close;
-    _gcount = 0;
+  init (sb, tied);
+  _gcount = 0;
 }
 
 int skip_ws(streambuf* sb)
@@ -69,6 +69,8 @@ istream& istream::get(char& c)
 	  _gcount = 1;
 	}
     }
+    else
+      _gcount = 0;
     return *this;
 }
 
@@ -86,13 +88,13 @@ int istream::peek()
 
 istream& istream::ignore(int n /* = 1 */, int delim /* = EOF */)
 {
+    _gcount = 0;
     if (ipfx1()) {
 	register streambuf* sb = _strbuf;
 	if (delim == EOF) {
 	    _gcount = sb->ignore(n);
 	    return *this;
 	}
-	_gcount = 0;
 	for (;;) {
 #if 0
 	    if (n != MAXINT) // FIXME
@@ -112,19 +114,36 @@ istream& istream::ignore(int n /* = 1 */, int delim /* = EOF */)
     return *this;
 }
 
-istream& istream::read(char *s, int n)
+istream& istream::read(char *s, streamsize n)
 {
     if (ipfx1()) {
 	_gcount = _strbuf->sgetn(s, n);
 	if (_gcount != n)
-	    set(ios::failbit);
+	    set(ios::failbit|ios::eofbit);
     }
+    else
+      _gcount = 0;
     return *this;
 }
 
+int
+istream::sync ()
+{
+  streambuf *sb = rdbuf ();
+  if (sb == NULL)
+    return EOF;
+  if (sb->sync ()) // Later: pubsync
+    { 
+      setstate (ios::badbit);
+      return EOF;
+    }
+  else
+    return 0;
+}
+
 istream& istream::seekg(streampos pos)
 {
-    pos = _strbuf->sseekpos(pos, ios::in);
+    pos = _strbuf->pubseekpos(pos, ios::in);
     if (pos == streampos(EOF))
 	set(ios::badbit);
     return *this;
@@ -132,10 +151,7 @@ istream& istream::seekg(streampos pos)
 
 istream& istream::seekg(streamoff off, _seek_dir dir)
 {
-  streampos pos
-    = _IO_seekoff (_strbuf, off,
-		   (_IO_seekflags)
-		   ((int)dir | _IO_seek_not_out | _IO_seek_pos_ignored));
+  streampos pos = _IO_seekoff (_strbuf, off, (int) dir, _IOS_INPUT);
   if (pos == streampos(EOF))
     set(ios::badbit);
   return *this;
@@ -144,11 +160,9 @@ istream& istream::seekg(streamoff off, _seek_dir dir)
 streampos istream::tellg()
 {
 #if 0
-    streampos pos = _strbuf->sseekoff(0, ios::cur, ios::in);
+    streampos pos = _strbuf->pubseekoff(0, ios::cur, ios::in);
 #else
-    streampos pos
-      = _IO_seekoff (_strbuf, 0,
-		     (_IO_seekflags)(_IO_seek_cur | _IO_seek_not_out));
+    streampos pos = _IO_seekoff (_strbuf, 0, _IO_seek_cur, _IOS_INPUT);
 #endif
     if (pos == streampos(EOF))
 	set(ios::badbit);
@@ -167,35 +181,33 @@ istream& istream::operator>>(char& c)
     return *this;
 }
 
-istream& istream::operator>>(char* ptr)
+istream&
+istream::operator>> (char* ptr)
 {
   register char *p = ptr;
   int w = width(0);
-  if (ipfx0()) {
-    register streambuf* sb = _strbuf;
-    for (;;)
-      {
-	int ch = sb->sbumpc();
-	if (ch == EOF)
-	  {
-	    set(p == ptr ? (ios::eofbit|ios::failbit) : (ios::eofbit));
-	    break;
-	  }
-	else if (isspace(ch))
-	  {
-	    sb->sputbackc(ch);
-	    break;
-	  }
-	else if (w == 1)
-	  {
-	    set(ios::failbit);
-	    sb->sputbackc(ch);
-	    break;
-	  }
-	else *p++ = ch;
-	w--;
-      }
-  }
+  if (ipfx0())
+    {
+      register streambuf* sb = _strbuf;
+      for (;;)
+	{
+	  int ch = sb->sbumpc();
+	  if (ch == EOF)
+	    {
+	      set(ios::eofbit);
+	      break;
+	    }
+	  else if (isspace(ch) || w == 1)
+	    {
+	      sb->sputbackc(ch);
+	      break;
+	    }
+	  else *p++ = ch;
+	  w--;
+	}
+      if (p == ptr)
+	set(ios::failbit);
+    }
   *p = '\0';
   return *this;
 }
@@ -302,6 +314,16 @@ READ_INT(unsigned long)
 READ_INT(long long)
 READ_INT(unsigned long long)
 #endif
+#if _G_HAVE_BOOL
+READ_INT(bool)
+#endif
+
+istream& istream::operator>>(long double& x)
+{
+    if (ipfx0())
+	scan("%lg", &x);
+    return *this;
+}
 
 istream& istream::operator>>(double& x)
 {
@@ -390,7 +412,7 @@ static void write_int(ostream& stream, unsigned LONGEST val, int sign)
 	    *--buf_ptr = (val & 7) + '0';
 	    val = val >> 3;
 	} while (val != 0);
-	if ((stream.flags() & ios::showbase) && (val != 0))
+	if ((stream.flags() & ios::showbase) && (*buf_ptr != '0'))
 	    *--buf_ptr = '0';
     }
     else if ((stream.flags() & ios::basefield) == ios::hex) { // Hex
@@ -400,7 +422,7 @@ static void write_int(ostream& stream, unsigned LONGEST val, int sign)
 	    *--buf_ptr = xdigs[val & 15];
 	    val = val >> 4;
 	} while (val != 0);
-	if ((stream.flags() & ios::showbase) && (val != 0)) {
+	if ((stream.flags() & ios::showbase)) {
 	    show_base = xdigs + 16; // Either "0X" or "0x".
 	    show_base_len = 2;
 	}
@@ -472,9 +494,10 @@ ostream& ostream::operator<<(int n)
 {
     if (opfx()) {
 	int sign = 1;
+	unsigned int abs_n = (unsigned)n;
 	if (n < 0 && (flags() & (ios::oct|ios::hex)) == 0)
-	    n = -n, sign = -1;
-	write_int(*this, n, sign);
+	    abs_n = -((unsigned)n), sign = -1;
+	write_int(*this, abs_n, sign);
     }
     return *this;
 }
@@ -491,9 +514,9 @@ ostream& ostream::operator<<(long n)
 {
     if (opfx()) {
 	int sign = 1;
-	unsigned int abs_n = (unsigned)n;
+	unsigned long abs_n = (unsigned long)n;
 	if (n < 0 && (flags() & (ios::oct|ios::hex)) == 0)
-	    abs_n = -((unsigned)n), sign = -1;
+	    abs_n = -((unsigned long)n), sign = -1;
 	write_int(*this, abs_n, sign);
     }
     return *this;
@@ -535,9 +558,6 @@ ostream& ostream::operator<<(double n)
 	// stdio implementation.  The setup code uses the same logic
 	// as in __vsbprintf.C (also based on Torek's code).
 	int format_char;
-#if 0
-	if (flags() ios::showpos) sign = '+';
-#endif
 	if ((flags() & ios::floatfield) == ios::fixed)
 	    format_char = 'f';
 	else if ((flags() & ios::floatfield) == ios::scientific)
@@ -545,22 +565,24 @@ ostream& ostream::operator<<(double n)
 	else
 	    format_char = flags() & ios::uppercase ? 'G' : 'g';
 
-	int fpprec = 0; // 'Extra' (suppressed) floating precision.
 	int prec = precision();
+	if (prec <= 0 && !(flags() & ios::fixed))
+	  prec = 6; /* default */
+
+	// Do actual conversion.
+#ifdef _IO_USE_DTOA
+	if (_IO_outfloat(n, rdbuf(), format_char, width(0),
+			 prec, flags(),
+			 flags() & ios::showpos ? '+' : 0,
+			 fill()) < 0)
+	    set(ios::badbit|ios::failbit); // ??
+#else
+	int fpprec = 0; // 'Extra' (suppressed) floating precision.
 	if (prec > MAXFRACT) {
 	    if (flags() & (ios::fixed|ios::scientific) & ios::showpos)
 		fpprec = prec - MAXFRACT;
 	    prec = MAXFRACT;
 	}
-	else if (prec <= 0 && !(flags() & ios::fixed))
-	  prec = 6; /* default */
-
-	// Do actual conversion.
-#ifdef USE_DTOA
-	if (_IO_outfloat(n, rdbuf(), format_char, width(0),
-		       prec, flags(), 0, fill()) < 0)
-	    set(ios::badbit|ios::failbit); // ??
-#else
 	int negative;
 	char buf[BUF];
 	int sign = '\0';
@@ -571,6 +593,7 @@ ostream& ostream::operator<<(double n)
 				&negative,
 				format_char, cp, buf + sizeof(buf));
 	if (negative) sign = '-';
+	else if (flags() & ios::showpos) sign = '+';
 	if (*cp == 0)
 	    cp++;
 
@@ -664,14 +687,14 @@ ostream& ostream::operator<<(register streambuf* sbuf)
   return *this;
 }
 
-ostream::ostream(streambuf* sb, ostream* tied) : ios(sb, tied)
+ostream::ostream(streambuf* sb, ostream* tied)
 {
-    _flags |= ios::dont_close;
+  init (sb, tied);
 }
 
 ostream& ostream::seekp(streampos pos)
 {
-    pos = _strbuf->sseekpos(pos, ios::out);
+    pos = _strbuf->pubseekpos(pos, ios::out);
     if (pos == streampos(EOF))
 	set(ios::badbit);
     return *this;
@@ -679,10 +702,7 @@ ostream& ostream::seekp(streampos pos)
 
 ostream& ostream::seekp(streamoff off, _seek_dir dir)
 {
-  streampos pos
-    = _IO_seekoff (_strbuf, off,
-		   (_IO_seekflags)
-		   ((int)dir | _IO_seek_not_in | _IO_seek_pos_ignored));
+  streampos pos = _IO_seekoff (_strbuf, off, (int) dir, _IOS_OUTPUT);
   if (pos == streampos(EOF))
     set(ios::badbit);
   return *this;
@@ -691,11 +711,9 @@ ostream& ostream::seekp(streamoff off, _seek_dir dir)
 streampos ostream::tellp()
 {
 #if 1
-    streampos pos
-      = _IO_seekoff (_strbuf, 0,
-		     (_IO_seekflags)(_IO_seek_cur | _IO_seek_not_in));
+    streampos pos = _IO_seekoff (_strbuf, 0, _IO_seek_cur, _IOS_OUTPUT);
 #else
-    streampos pos = _strbuf->sseekoff(0, ios::cur, ios::out);
+    streampos pos = _strbuf->pubseekoff(0, ios::cur, ios::out);
 #endif
     if (pos == streampos(EOF))
 	set(ios::badbit);
@@ -704,7 +722,7 @@ streampos ostream::tellp()
 
 ostream& ostream::flush()
 {
-    if (_strbuf->_jumps->__sync(_strbuf))
+    if (_strbuf->sync())
 	set(ios::badbit);
     return *this;
 }
@@ -754,7 +772,7 @@ ostream& endl(ostream& outs)
     return flush(outs.put('\n'));
 }
 
-ostream& ostream::write(const char *s, int n)
+ostream& ostream::write(const char *s, streamsize n)
 {
     if (opfx()) {
 	if (_IO_sputn(_strbuf, s, n) != n)
@@ -773,9 +791,9 @@ void ostream::do_osfx()
     }
 }
 
-iostream::iostream(streambuf* sb, ostream* tied) : ios(sb, tied)
+iostream::iostream(streambuf* sb, ostream* tied)
 {
-  _flags |= ios::dont_close;
+  init (sb, tied);
 }
 
 // NOTE: extension for compatibility with old libg++.
@@ -783,13 +801,10 @@ iostream::iostream(streambuf* sb, ostream* tied) : ios(sb, tied)
 #ifdef _STREAM_COMPAT
 void ios::close()
 {
-  if (!(_flags & (unsigned int)ios::dont_close))
-    delete rdbuf();
-  else if (_strbuf->_flags & _IO_IS_FILEBUF)
+  if (_strbuf->_flags & _IO_IS_FILEBUF)
     ((struct filebuf*)rdbuf())->close();
   else if (_strbuf != NULL)
     rdbuf()->sync();
-  _flags |= ios::dont_close;
   _strbuf = NULL;
   _state = badbit;
 }
